#include <windows.h>
#include "Hook.hpp"


static LRESULT CALLBACK MainWindowProc(HWND window, UINT message, WPARAM wparam, LPARAM lparam);


////////////////////////////////////////////////////////////////////////////////

int WINAPI WinMain(HINSTANCE instance, HINSTANCE, LPSTR, int)
{
  // check to see if program is already running
  HANDLE mutex = CreateMutex(NULL, FALSE, "Minnie");
  if (mutex == NULL) {  // creation failed
    return 0;
  }

  // mutex already exists
  if (GetLastError() == ERROR_ALREADY_EXISTS) {
    // tell program to die
    SendMessage(FindWindow("MinnieClass", NULL), WM_CLOSE, 0, 0);
    return 0;
  }

  // register main window class
  WNDCLASS wc;
  memset(&wc, 0, sizeof(wc));
  wc.lpfnWndProc = MainWindowProc;
  wc.hInstance = instance;
  wc.hIcon = NULL;
  wc.hCursor = LoadCursor(NULL, MAKEINTRESOURCE(IDC_ARROW));
  wc.hbrBackground = NULL;
  wc.lpszMenuName = NULL;
  wc.lpszClassName = "MinnieClass";
  if (RegisterClass(&wc) == 0) {
    MessageBox(NULL, "Window class registration failed", "Minnie", MB_OK | MB_ICONERROR);
    return -1;
  }

  // create hidden main window
  HWND window = CreateWindow(
    wc.lpszClassName,
    "Minnie",
    WS_POPUP,
    CW_USEDEFAULT, 0,
    CW_USEDEFAULT, 0,
    NULL,
    NULL,
    instance,
    NULL);
  if (window == NULL) {
    MessageBox(NULL, "Window creation failed", "Minnie", MB_OK | MB_ICONERROR);
    return -1;
  }

  if (!SetHook()) {
    MessageBox(NULL, "Could not set keyboard hook", "Minnie", MB_OK | MB_ICONERROR);
    return -1;
  }

  // message loop
  MSG msg;
  while (GetMessage(&msg, NULL, 0, 0) > 0) {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }

  RemoveHook();

  CloseHandle(mutex);
  return msg.wParam;
}

////////////////////////////////////////////////////////////////////////////////

LRESULT CALLBACK MainWindowProc(HWND window, UINT message, WPARAM wparam, LPARAM lparam)
{
  switch (message) {

    case WM_CLOSE: {
      PostQuitMessage(0);
      return 0;
    }

    // middle mouse down
    case WM_APP + 0: {
      INPUT inputs[3];
      inputs[0].type = INPUT_MOUSE;
      inputs[0].mi.dx          = wparam;
      inputs[0].mi.dy          = lparam;
      inputs[0].mi.mouseData   = 0;
      inputs[0].mi.dwFlags     = MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_LEFTDOWN;
      inputs[0].mi.time        = 0;
      inputs[0].mi.dwExtraInfo = NULL;

      inputs[1] = inputs[0];
      inputs[2] = inputs[0];

      inputs[1].mi.dwFlags = MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_LEFTUP;

      SendInput(3, inputs, sizeof(INPUT));

      return 0;
    }

    // middle mouse up
    case WM_APP + 1: {
      INPUT input;
      input.type = INPUT_MOUSE;
      input.mi.dx          = wparam;
      input.mi.dy          = lparam;
      input.mi.mouseData   = 0;
      input.mi.dwFlags     = MOUSEEVENTF_ABSOLUTE | MOUSEEVENTF_LEFTUP;
      input.mi.time        = 0;
      input.mi.dwExtraInfo = NULL;

      SendInput(1, &input, sizeof(INPUT));

      return 0;
    }

    default: {
      return DefWindowProc(window, message, wparam, lparam);
    }
  }
}

////////////////////////////////////////////////////////////////////////////////
